 ; -----------------------------------------
; NTP91V1 Steuer-CPU fr Thyristor-Vorregler
; 600W-Labornetzgert 
; -----------------------------------------

.nolist
	.include "tn25def.inc"
	.include "allgdef.inc"
.list

.CSEG

	.org	0x0000
	rjmp	restart

	.org	0x0004
	rjmp		tim1int		; timer1 Interrupt

	.org	0x100
	
	
restart:
	cli 

; Stack
	ldi	acc,low(RAMEND) - 2
	out	spl,acc
	ldi	acc,high(RAMEND) - 2
	out	sph,acc
	
; Ports einstellen

	ldi	acc2,0b00000000
	ldi	acc,0b00011010
	out	PORTB,acc2
	mov		outbyte,acc2
	out	DDRB,acc
	; Insert nop for synchronization
	nop
	
; Wartezeit

        ldi     acc,255
warten:
        ldi     acc2,255
warten2:
        nop
        nop
        nop
        nop
        dec     acc2
        brne    warten2
        dec     acc
        brne    warten     
	
; Analog-Eingang ADC1

	ldi	acc,0b00100001	; AVCC=Ref, links (8bit in ADCH), ADC1 = Tasten
	out	admux,acc
	ldi	acc,0b00000101	; 8MHz:32=250kHz, 1.Wandlung dauert = 100us
	out	adcsra,acc
	
; Timer1
	ldi		acc,0b00000011 	; CLK / 64
	out		tccr1,acc
	ldi		acc,0b00000100	; berlauf-Int
	out		tifr,acc
	ldi		acc,0b00000100
	out		timsk,acc


; Anfangswert
	ldi		acc,20			; 2 sek
	sts		bettim,acc

	
;--------------------------------------------------
; HP  *********************************************
;--------------------------------------------------


hp:
	rcall	anainp
	rcall	timer
	rcall	betrieb
	rjmp	hp

; -----------------------------------------------------
; BETRIEB
; -----------------------------------------------------
betrieb:
	sbis	pinb,0
	rjmp	bet1
	ldi		acc,20			; 2 sek
	sts		bettim,acc
	ret
bet1:
	lds		acc,bettim
	tst		acc
	breq	bet2
	clr		acc
	sts		TastenAna+2,acc
	ret
bet2:
	lds		acc,TastenAna+2
	sbrc	acc,0
	rjmp	bet3	
	sbrc	acc,1
	rjmp	bet4
	sbrc	acc,2
	rjmp	bet5
	ret
; Taste 1
bet3:
	sbrc	outbyte,1
	rjmp	bet3a
	ori		outbyte,0b00000010
	rjmp	bet6
bet3a:
	andi	outbyte,0b11111101
	rjmp	bet6
; Taste 2
bet4:
	sbrc	outbyte,3
	rjmp	bet4a
	ori		outbyte,0b00001000
	rjmp	bet6
bet4a:
	andi	outbyte,0b11110111
	rjmp	bet6
; Taste 3
bet5:
	sbrc	outbyte,4
	rjmp	bet5a
	ori		outbyte,0b00010000
	rjmp	bet6
bet5a:
	andi	outbyte,0b11101111
; Ausgabe
bet6:
	out		portb,outbyte
	clr		acc
	sts		TastenAna+2,acc
	ret

; -----------------------------------------------------
; TIMER
; -----------------------------------------------------

timer:
	cli
	mov		acc2,softtimer 		; 100us-Takte
	subi	acc2,10				; fr 1ms
	brcs	timer0
	sei
	ret
timer0:
	mov		softtimer,acc2 		; Rest zurck
	sei

; 1ms-Takte
	
	lds		acc,etptimer
	tst		acc
	breq	timer1
	dec		acc
	sts		etptimer,acc
timer1:
	lds		acc,vortimer
	tst		acc
	breq	timer2
	dec		acc
	sts		vortimer,acc
	ret
timer2:
	ldi		acc,100 			; fr 100ms
	sts		vortimer,acc
	lds		acc,bettim
	tst		acc
	breq	timer3
	dec		acc
	sts		bettim,acc
timer3:
	ret


; -----------------------------------------------------
; ANAINP
; -----------------------------------------------------
; Ergebnis in (D)TastenAna+2
; b0= Taste 1
; b1= Taste 2
; b2= Taste 3

anainp:       
	sbic	adcsra,adsc	; ADW fertig?
	rjmp	ana2		; nein
	in		acc,adcl
	in		acc,adch
	ldi		yl,low(TastenAna)
	ldi		yh,high(TastenAna)
	rcall	entprell		; acc -> RAM
	ldi		acc,0b11000000	; startet Single Conversation
	out		ADCSRA,acc
ana2:
	ret
	
;--------------------------------------------------
; entprell
;--------------------------------------------------
; Eingang: Analogwert im acc, Adresse in y
; Organisation im RAM:
; +0 Entprellkopie
; +1 Bitmusterkopie, ntig um eine neues Muster zu erkennen
; +2 Bitmuster-Ausgabe, wird nur 1 mal berschrieben

entprell:
	cpi		acc,205
	brcs	entprell3
	ldi		acc,1
	rjmp		entprell6
entprell3:
	cpi		acc,118
	brcs	entprell4
	ldi		acc,2
	rjmp		entprell6
entprell4:
	cpi		acc,36
	brcs	entprell5
	ldi		acc,4
	rjmp		entprell6
entprell5:
	clr		acc
; Bitmuster im acc
entprell6:
	ld		acc2,y+		; Eingangs-Bitmuster von vorher
	cp		acc,acc2	; gendert?
	breq	entprell7	; nein			; nein
	st		-y,acc		; ja, bernehmen!
	ldi		acc,40		; 40ms
	sts		etptimer,acc
	ret
entprell7:
	lds		acc3,etptimer
	tst		acc3		; Entprellzeit vorbei?
	breq	entprell8	; ja
	ret			; nein
entprell8:
	ld		acc2,y
	cp		acc,acc2	; neues Muster?
	brne	entprell9
	ret			; nein
entprell9:
	st		y+,acc
	st		y,acc		; Ausgabe
	ret
	

	nop
	nop
	reti

; ------------------------------------------------
; Interrupt
; ------------------------------------------------
; Timer1 OVF alle 0,5x200us = 100us berlauf-Int

tim1int:
	push	acc
	in		acc,sreg
	push	acc
	inc		softtimer
	ldi		acc,59
	out		tcnt1,acc
	pop		acc
	out		sreg,acc
	pop		acc
	reti

